; Convert a VGA video buffer to GP32 format
;  by Jeff F, 2002-Apr-4
; 
; Scherzo (Paul W.) on 2003-Apr-22 did the following:
; - Converted routine for ARM ADS assembler
; - Added a 16 bit version of the routine
;
;  These ARM assembly routines convert an 8-bit or 16-bit VGA style
; video buffer to GP32 8-bit or 16-bit video frame buffer format.
;
;  The default Performance is set to 4. The smallest code
; is obtained by setting this value to 1. The fastest and
; largest code is obtained by setting this value to 80
; (160 for the 16-bit version).
; Here are the only valid values for Performance:
;   1,2,4,5,8,10,16,20,40,80,(160)
; The percentage gain of speed improvements drops off
; exponentially with each larger Performance setting
; so 4 is probably fine for most cases.
;
; Reference them from C by using:
;  extern void XlatVGABuffer (unsigned char *src, unsigned char *dest);
;  extern void XlatVGABuffer16 (unsigned short* src, unsigned short* dest);

        AREA    Xlat, CODE, READONLY

Performance			EQU	80
Performance16		EQU 160
Performance16Nester	EQU	128

        EXPORT XlatVGABuffer
XlatVGABuffer	PROC
        ; r0 points to source
        ; r1 points to destination
        stmfd r13!,{r4}
        mov r12,#240
        add r1,r1,#240
XlatLoop1
		sub r1,r1,#1
		mov r3,r1
		mov r4,#80/Performance
XlatLoop2
		GBLA	count
count	SETA	0
		WHILE	count < Performance
count	SETA	count+1
		ldr r2,[r0],#4
		strb r2,[r3],#240
		mov r2,r2,lsr #8
		strb r2,[r3],#240
		mov r2,r2,lsr #8
		strb r2,[r3],#240
		mov r2,r2,lsr #8
		strb r2,[r3],#240
		WEND
		
		subs r4,r4,#1
		bne XlatLoop2

		subs r12,r12,#1
		bne XlatLoop1
        
        ldmfd r13!,{r4}
        ENDP

        AREA    Xlat16, CODE, READONLY

		EXPORT XlatVGABuffer16
XlatVGABuffer16	PROC
		; r0 points to source
		; r1 points to destination
		stmfd r13!,{r4}
		mov r12,#240
		add r1,r1,#480
Xlat16Loop1
		sub r1,r1,#2
		mov r3,r1
		mov r4,#160/Performance16
Xlat16Loop2
		GBLA	count
count	SETA	0
		WHILE	count < Performance16
count	SETA	count+1
		ldr r2,[r0],#4
		strh r2,[r3],#240
		add r3,r3,#240
		mov r2,r2,lsr #16
		strh r2,[r3],#240
		add r3,r3,#240
		WEND
		
		subs r4,r4,#1
		bne Xlat16Loop2

		subs r12,r12,#1
		bne Xlat16Loop1

		ldmfd r13!,{r4}
		ENDP

		END
